--[[---------------------------------------------------------------------------------------------
MountSafe Emotes

Wildstar has many animated emotes. Like other MMOs, these both animate the character and display
some emote text in the chat log. Like many other MMOs, Wildstar cannot display the animations
when the character is mounted. Unlike other MMOs, Wildstar doesn't even bother to output the chat
text if the character is mounted. This means, for example, that typing /wave (or using a hotkey
bound to a macro that says "/wave") will do *nothing* if you're on a ground mount or hoverboard,
even though the emote text would work fine (and, in the case of /wave, the upper body animation
would work fine on a hoverboard).

MountSafe Emotes introduces a new command "/mountsafe" (and its shorter alias, "/ms"), which
takes one argument: the name of an animated emote. If the character is not mounted, the emote
is deferred to the standard command interpreter, which causes the whole shebang of animating
and displaying text to happen. If the character *is* mounted, MountSafe spits out some emote
text which is very similar to what the command interpreter would do.

Why "very similar"? The built-in command interpreter (ChatSystemLib, a hosting API whose details
are not visible to us in the scripting world) can do at least two things we can't: It can display
emote text with arbitrary syntax, whereas we have to use the form "<character name> <arbitrary text>",
and it can display different text to other people ("Reys waves.") than it does to the emoting
character ("You wave."). Consequently, MountSafe can only approximate the output.

The output text needs to vary in two ways: If the user has a target, many of the emotes use the
target name, and some emotes need the emoting character's gender to form appropriate pronouns.
It is possible that some emotes would need the target's gender, but none of the existing emotes
as of 1.0.9.6780 require this.

Since the character's gender is used, Chua present some problems. The two obvious answers are
to use the neuter possessive pronoun ("its") or (and this is my preference) the singular neuter
plural("their"). However, Wildstar's buit-in emotes use the masculine ("his"), so that's what
MountSafe does.

-----------------------------------------------------------------------------------------------
This is some text to change the file hash. Curse is stupid and won't let me update the release notes, file name, and release type (Beta to Release) without changing the hash, because they're basically idiots.

Changelog
    * 1.3 - 2014.11.07 - Initial French translation. Much thanks to Curse user Umbura!
                       - Added option to intercept emotes while mounted so you don't have to use the slash command
    * 1.2 - 2014.10.21 - Cleaned up the spam on startup
                       - Moved some string literals to make translation easier
    * 1.1 - 2014.09.28 - First release version.
                       - Name changed from "MountSafe" to "MountSafe Emotes"
                       - General prettying up so I can make changes later if I haven't seen the
                         code in a while.
    * 1.0 - 2014.07.27 - Initial beta upload, accidentally tagged as 1.0 instead of 0.9
----------------------------------------------------------------------------------------------- --]]

require "ChatSystemLib"

-----------------------------------------------------------------------------------------------
-- MountSafe Module Definition
-----------------------------------------------------------------------------------------------
local MountSafe = {}

-----------------------------------------------------------------------------------------------
-- Constants
local kVersion = "1.3"

-- GameLib.GetPlayerUnit():GetGender() returns kGender.Male or kGender.Female
local kGender = {Male = 0, Female = 1, Chua = 2}

-- Magic number for Chua for GetRaceId()
local kRaceChua = 13

-- Constants for replacement strings make the string literals longer, but easier to read.
local kRep =
{
    nl  = "\n",	-- Newline constant
    tar = "$t", -- Target name
    pos = "$p", -- Possessive pronoun replacement token
}



-----------------------------------------------------------------------------------------------
-- Localization. Kind of a cheesey approach, but our needs are pretty simple
-- Much thanks to Umbura for the French translations!
-----------------------------------------------------------------------------------------------

--region GeminiLocale
    ---------------------------------------------------------------------------
    -- This locale detection code is lifted from GeminiLocale
    ---------------------------------------------------------------------------
    -- this locale lookup table is accurate for build 6512
    ktLocales = 
    {
        English = "enUS",
        German  = "deDE",
        French  = "frFR",
        KR      = "koKR",
    }

    -- access to read the locale.languageId console variable has been disabled
    -- check what the Apollo.GetString(1) (aka "Cancel") is translated into and 
    -- return the language locale
    local function GetLocale(self)
        local strCancel = Apollo.GetString(1)
    
        -- German
        if strCancel == "Abbrechen" then 
            return ktLocales.German
        end
    
        -- French
        if strCancel == "Annuler" then
            return ktLocales.French
        end
    
        -- Other
        return ktLocales.English
    --	return ktLocales[(Apollo.GetConsoleVariable("locale.languageId") or 1)]
    end
--endregion GeminiLocale
    
local Locales = 
{
    [ktLocales.English] = 
    {
        Apply = function(self)
            return {Commands = self.kCommands, Messages = self.kMessages, Emotes = self.ktEmotes, Pronouns = self.kPronouns}
        end,

        kPronouns = {Possessive = {[kGender.Male] = "his", [kGender.Female] = "her", [kGender.Chua] = "their"}},

        kCommands =
        {
            Prefix = String_GetWeaselString(Apollo.GetString("ChatLog_SlashPrefix")),
            Emote  = "emote",
            Hook   = 
            {
                Command = "intercept",
                On      = "on",
                Off     = "off",
            },
        },
            
        kMessages =
        {
            Name         = "MountSafe Emotes",
            Tagline      = "Emotes that work even when you're mounted.",
            Version      = "Version "..kVersion,
            Usageline    = "Usage: '/ms <emote>'. Use '/ms' by itself for a list of supported emotes.",
            Supported    = "Supported emotes",

            InterceptStatusOn  = "Intercepting normal emotes is on. Use '/ms intercept off' to disable.",
            InterceptStatusOff = "Intercepting normal emotes is off. Use '/ms intercept on' to enable.",
            InterceptUsage = "",

            UnknownEmote = "Unknown emote",
        },
        
        --[[
            An emote is:
                name			(string)
                mountedtext 	(table)
                    targeted	(string)
                    untargeted	(string)

            Replacements:
                $t 	Target name
                $p 	Player's possessive pronoun (his/her/their/its)
        --]]
        ktEmotes =
        {
            no       = {targeted = "shakes "..kRep.pos.." head at "..kRep.tar..". No!",         untargeted = "shakes "..kRep.pos.." head. No!"},
            yes      = {targeted = "nods "..kRep.pos.." head at "..kRep.tar..". Yes!",          untargeted = "nods "..kRep.pos.." head. Yes!"},

            applaud  = {targeted = "applauds for "..kRep.tar.."",                               untargeted = "applauds"},
            clap     = {targeted = "claps "..kRep.pos.." hands for "..kRep.tar.."",             untargeted = "claps "..kRep.pos.." hands."},

            cheer    = {targeted = "cheers for "..kRep.tar.."!",                                untargeted = "cheers!"},
            hurray   = {targeted = "cheers for "..kRep.tar.."!",                                untargeted = "cheers!"},

            cry      = {targeted = "cries at "..kRep.tar.."",                                   untargeted = "cries."},
            crying   = {targeted = "takes a moment.",                                           untargeted = "takes a moment."},

            dance    = {targeted = "dances spryly for "..kRep.tar..".",                         untargeted = "dances spryly."},
            dance1   = {targeted = "dances spryly for "..kRep.tar..".",                         untargeted = "dances spryly."},
            dance2   = {targeted = "dances feverishly for "..kRep.tar..".",                     untargeted = "dances feverishly."},

            cower    = {targeted = "cowers in fear of "..kRep.tar..".",                         untargeted = "cowers in fear."},
            fear     = {targeted = "cowers in fear of "..kRep.tar..".",                         untargeted = "cowers in fear."},

            farewell = {targeted = "waves farewell to "..kRep.tar..".",                         untargeted = "waves farewell."},
            bye      = {targeted = "waves goodbye to "..kRep.tar..".",                          untargeted = "waves goodbye."},
            goodbye  = {targeted = "waves goodbye to "..kRep.tar..".",                          untargeted = "waves goodbye."},

            shout    = {targeted = "shouts at "..kRep.tar..".",                                 untargeted = "shouts with vigor."},
            talk     = {targeted = "casually chats with "..kRep.tar..".",                       untargeted = "casually chats with everyone."},
            flex     = {targeted = "gives "..kRep.tar.." a ticket to the gun show.",            untargeted = "gives everyone a ticket to the gun show."},
            kneel    = {targeted = "kneels before "..kRep.tar..".",                             untargeted = "kneels respectfully."},
            point    = {targeted = "points at "..kRep.tar..".",                                 untargeted = "points off into the distance."},
            shrug    = {targeted = "shrugs at "..kRep.tar..".",                                 untargeted = "shrugs."},
            wave     = {targeted = "waves to "..kRep.tar..".",                                  untargeted = "waves."},
            beckon   = {targeted = "beckons to "..kRep.tar..". Over here!",                     untargeted = "beckons to everyone. Over here!"},
            bow      = {targeted = "bows to "..kRep.tar..".",                                   untargeted = "bows."},
            laugh    = {targeted = "laughs at "..kRep.tar..".",                                 untargeted = "laughs."},
            salute   = {targeted = "salutes "..kRep.tar..".",                                   untargeted = "salutes."},
            sleep    = {targeted = "is bored into unconsciousness by "..kRep.tar..".",          untargeted = "takes a nap."},
            drink    = {targeted = "raises "..kRep.pos.." glass to "..kRep.tar..". Cheers!",    untargeted = "raises "..kRep.pos.." glass to everyone. Cheers!"},
            eat      = {targeted = "teases "..kRep.tar.." with "..kRep.pos.." delicious food.", untargeted = "takes a big scrumptious bite of food. Yum!"},
            stand    = {targeted = "stands up before "..kRep.tar..".",                          untargeted = "stands up."},
            sit      = {targeted = "sits with "..kRep.tar..".",                                 untargeted = "sits down."},
            nod      = {targeted = "nods "..kRep.pos.." head at "..kRep.tar..".",               untargeted = "nods "..kRep.pos.." head."},

            strain   = {targeted   = "begs "..kRep.tar.." for help as a Strain creature erupts violently from "..kRep.pos.." chest!",
                        untargeted = "begs for help as a Strain creature erupts violently from "..kRep.pos.." chest!"},
        },

    },

    [ktLocales.French] = 
    {
        -- French doesn't need kPronouns

        kMessages = 
        {
            Name      = "MountSafe Emotes",
            Tagline   = "MountSafe Emotes: des emotes qui fonctionnent mme sur votre monture !",
            Version   = "Version "..kVersion,
            Usageline = "Utilisation : '/ms <emote>'. Lancez '/ms' sans paramtres pour obtenir la liste des emotes supports.",
            Supported = "", --TODO: Get French version of this

            InterceptStatusOn  = "Interception normale motes est sur. Utilisez  /ms intercept teint  pour dsactiver.",
            InterceptStatusOff = "Interception normale motes est teint. Utilisation  /ms intercept sur  pour activer.",
            InterceptUsage     = "",

            UnknownEmote = "Unknown emote",
        },

        kCommands = 
        {
            Prefix = String_GetWeaselString(Apollo.GetString("ChatLog_SlashPrefix")),
            Emote  = "mote",
            Hook   = 
            {
                Command = "intercepter",    --TODO: Verify!
                On      = "sur",            --TODO: Verify!
                Off     = "teint",         --TODO: Verify!
            },
        },

        ktEmotes = 
        {
            non      = {targeted = "secoue la tte en regardant "..kRep.tar..". Non !",         untargeted = "secoue la tte. Non !"},
            oui      = {targeted = "hoche la tte en regardant "..kRep.tar..". Oui !",          untargeted = "hoche la tte. Oui !"},
    
            applaudir = {targeted = "applaudit "..kRep.tar..".",                                untargeted = "applaudit"},
            clap = {targeted = "applaudit "..kRep.tar..".",                                     untargeted = "applaudit"},
    
            acclamer = {targeted = "acclame "..kRep.tar.." !",                                  untargeted = "pousse une acclamation !"},
            hourra   = {targeted = "acclame "..kRep.tar.." !",                                  untargeted = "pousse une acclamation !"},
    
            pleurer  = {targeted = "pleure en regardant "..kRep.tar.."",                        untargeted = "pleure."},
            triste   = {targeted = "fait une pause pour se remettre de ses motions.",          untargeted = "fait une pause pour se remettre de ses motions."},
    
            danser   = {targeted = "danse allgrement pour "..kRep.tar..".",                    untargeted = "danse allgrement."},
            danser3  = {targeted = "danse allgrement pour "..kRep.tar..".",                    untargeted = "danse allgrement."},
            danser2  = {targeted = "danse frntiquement pour "..kRep.tar..".",                 untargeted = "danse frntiquement."},
    
            trembler = {targeted = "tremble de peur devant "..kRep.tar..".",                    untargeted = "tremble de peur."},
            peur     = {targeted = "tremble de peur devant "..kRep.tar..".",                    untargeted = "tremble de peur."},

            adieu    = {targeted = "fait un signe d'adieu  "..kRep.tar..".",                   untargeted = "fait un signe d'adieu."},
            salut    = {targeted = "salue "..kRep.tar.." de la main.",                          untargeted = "salue de la main."},
            aurevoir = {targeted = "salue "..kRep.tar.." de la main.",                          untargeted = "salue de la main."},
    
            -- shout    = {targeted = "shouts at "..kRep.tar..".",                                 untargeted = "shouts with vigor."},
            discuter = {targeted = "discute avec "..kRep.tar..".",                              untargeted = "discute avec tout le monde."},
            frimer   = {targeted = "roule des mcaniques devant "..kRep.tar..".",               untargeted = "roule des mcaniques devant tout le monde."},
            genoux   = {targeted = "s'agenouille devant "..kRep.tar..".",                       untargeted = "s'agenouille respectueusement."},
            montrer  = {targeted = "montre "..kRep.tar.." du doigt.",                           untargeted = "pointe le doigt vers l'horizon."},
            epaules  = {targeted = "hausse les paules en regardant "..kRep.tar..".",           untargeted = "hausse les paules."},
            salut    = {targeted = "salue "..kRep.tar.." de la main.",                          untargeted = "salue de la main."},
            signe    = {targeted = "fait signe  "..kRep.tar..". Par ici !",                    untargeted = "fait signe  tout le monde. Par ici !"},
            s_incliner = {targeted = "s'incline devant "..kRep.tar..".",                        untargeted = "s'incline."},
            rire     = {targeted = "clate de rire en regardant "..kRep.tar..".",               untargeted = "rit."},
            saluer   = {targeted = "salue "..kRep.tar..".",                                     untargeted = "salue."},
            dormir   = {targeted = "s'ennuie tellement avec "..kRep.tar.." qu'il s'endort.",    untargeted = "fait un somme."},
            boire    = {targeted = "lve son verre pour "..kRep.tar..". Sant !",               untargeted = "lve son verre pour tout le monde. Sant !"},
            manger   = {targeted = "tente "..kRep.tar.." avec sa nourriture dlicieuse.",       untargeted = "prend une grande bouche de nourriture succulente. Miam !"},
            debout   = {targeted = "se lve devant "..kRep.tar..".",                            untargeted = "se lve."},
            assis    = {targeted = "s'assoit devant "..kRep.tar..".",                           untargeted = "s'assoit."},
            hocher   = {targeted = "hoche la tte en regardant "..kRep.tar..".",               untargeted = "hoche la tte."},

            souillure = {targeted   = "supplie "..kRep.tar.." de l'aider alors qu'une crature de la souillure jaillit violement de sa poitrine !", 
                        untargeted = "appelle  l'aide alors qu'une crature de la souillure jaillit violement de sa poitrine !"},
        },
        
        Apply = function(self)
            --if Locales:GetLocale() ~= Locales.ktLocales.French then return; end

            return {Messages = self.kMessages, Commands = self.kCommands, Emotes = self.ktEmotes}
        end
    },
}

    
function MountSafe:SetLocale(lid)
    -- Do a default
    local tlDefault = Locales[ktLocales.English]:Apply()
    self.kCommands = tlDefault.Commands or {}
    self.kMessages = tlDefault.Messages or {}
    self.ktEmotes  = tlDefault.Emotes   or {}
    self.kPronouns = tlDefault.Pronouns or {}
    
    -- Overwrite with anything we need. 
    lid = lid or GetLocale()
    if Locales[lid] ~= nil then 
        local tlLID = Locales[lid]:Apply()
        self.kCommands = tlLID.Commands or self.kCommands 
        self.kMessages = tlLID.Messages or self.kMessages 
        self.ktEmotes  = tlLID.Emotes   or self.ktEmotes  
        self.kPronouns = tlLID.Pronouns or self.kPronouns 
    end

    self.tEmotes = {}
    for e, _ in pairs(self.ktEmotes) do table.insert(self.tEmotes, e) end
    table.sort(self.tEmotes)
end


-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function MountSafe:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    self.tEmotes = {}
    self:SetLocale()

    self.ShouldHook = true
    self.HasHooked  = false

    return o
end

function MountSafe:Init()
    local bHasConfigureFunction = false
    local strConfigureButtonText = ""
    local tDependencies = {"Gemini:Hook-1.0"}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

function MountSafe:OnSave(eLevel)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return; end
    return {ShouldHook = self.ShouldHook}
end

function MountSafe:OnRestore(eLevel, data)
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character or data == nil then return; end
    if data.ShouldHook ~= nil then self.ShouldHook = data.ShouldHook; end
end

-----------------------------------------------------------------------------------------------
-- MountSafe OnLoad, called by Apollo after RegisterAddon.
-----------------------------------------------------------------------------------------------
function MountSafe:OnLoad()
    -- Register handlers for events, slash commands and timer, etc.
    -- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
    Apollo.RegisterSlashCommand("mountsafe", "OnSlashMountSafe", self)
    Apollo.RegisterSlashCommand("ms",        "OnSlashMountSafe", self)
    Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)

    local GeminiHook = Apollo.GetPackage("Gemini:Hook-1.0").tPackage
    GeminiHook:Embed(self)
end

-----------------------------------------------------------------------------------------------
-- OnInterfaceMenuListHasLoaded
--TODO: Why are we using this event? Is there a better way to make sure the chat system is up and running?
-- Chose this event because it's what EntSpy uses and that always seems to work for me, but it is kind of
-- messy to have this just for the one line.
-----------------------------------------------------------------------------------------------
function MountSafe:OnInterfaceMenuListHasLoaded()
    self:HookChatLog()

    -- On load send a message to the chat log letting the user know it's running and reminding them what 
    -- the syntax is. Nothing too spammy.
    self:MessageOut(self.kMessages.Name..": ".. self.kMessages.Tagline.." "..self.kMessages.Usageline..kRep.nl..self:GetInterceptStatusText())
end

function MountSafe:HookChatLog()
    if self.ShouldHook and not self.HasHooked then
        local cl  = Apollo.GetAddon("ChatLog")
        local cli = Apollo.GetAddonInfo("ChatLog")
        if cl and cli and cli.bRunning == 1 then
            self:RawHook(cl, "OnChatInputReturn")
            self.HasHooked = true
        end
    end
end

function MountSafe:UnhookChatLog()
    if self.HasHooked then
        local cl  = Apollo.GetAddon("ChatLog")
        local cli = Apollo.GetAddonInfo("ChatLog")
        if cl and cli and cli.bRunning == 1 then
            self:Unhook(cl, "OnChatInputReturn")
            self.HasHooked = false
        end
    end
end

-- Checks for "/<emote>"
function MountSafe:IsSupportedEmote(strText)
    if string.sub(strText, 1, 1) ~= self.kCommands.Prefix then return false; end
    if self.ktEmotes[string.sub(strText, 2)] == nil then return false; end
    return true
end

function MountSafe:OnChatInputReturn(unk, wndHandler, wndControl, strText)
    if self:IsSupportedEmote(strText) and GameLib.GetPlayerUnit():IsMounted() then
        -- We are mounted and they used a non-MountSafe emote. Let's fix it
        strText = self.kCommands.Prefix.."ms "..string.sub(strText, 2)
    end
    
    self.hooks[Apollo.GetAddon("ChatLog")].OnChatInputReturn(unk, wndHandler, wndControl, strText)
end

function MountSafe:GetInterceptStatusText()
    if self.HasHooked == true then return self.kMessages.InterceptStatusOn; end
    return self.kMessages.InterceptStatusOff
end

function MountSafe:ShowInterceptStatus()
    self:MessageOut(self:GetInterceptStatusText())
end

-----------------------------------------------------------------------------------------------
-- MountSafe Command Line
-- Slash Command: "/mountsafe" or "/ms"
-----------------------------------------------------------------------------------------------
function MountSafe:OnSlashMountSafe(strCommand, strEmote)
    -- If the user entered just "/ms", show some help and bail out.
    if strEmote == nil or strEmote == "" then
        self:ShowHelp()
        return
    end

    -- Special commands:
    -- /ms intercept        Reports intercept status and information
    -- /ms intercept off    Turns interception (hooking the chat log) off
    -- /ms intercept on     Turns interception (hooking the chat log) on
    if string.match(strEmote, "^%s*"..self.kCommands.Hook.Command.."%s*$") then
        self:ShowInterceptStatus()
        return
    end

    -- Enable interception
    if string.match(strEmote, "^%s*"..self.kCommands.Hook.Command.."%s+"..self.kCommands.Hook.On.."%s*$") then
        self.ShouldHook = true
        self:HookChatLog()
        self:ShowInterceptStatus()
        return
    end

    -- Disable interception
    if string.match(strEmote, "^%s*"..self.kCommands.Hook.Command.."%s+"..self.kCommands.Hook.Off.."%s*$") then
        self.ShouldHook = false
        self:UnhookChatLog()
        self:ShowInterceptStatus()
        return
    end

    -- Special testing commands
    if string.match(strEmote, "^%s*setlocale%s*$") then
        self:SetLocale()
        return
    end

    -- With a real regex available, we could do this in one step, but Lua chooses a smaller, faster matching system
    if string.match(strEmote, "^%s*setlocale%s+%w+%s*$") then
        local locale = string.match(strEmote, "^%s*setlocale%s+(%w+)%s*$")
        Print("Setting locale to '"..locale.."'")
        self:SetLocale(locale)
        self.kCommands.Emote = "emote" -- Have to use the EN command for testing
        return
    end
    

    -- Okay, get the rest of the command tail. First, clean it up so "/ms wave " works (trim trailing whitespace)
    strEmote = self:Trim(strEmote)

    -- The emotes table (self.ktEmotes) is keyed on the emote name.
    local emote = self.ktEmotes[strEmote]
    if emote ~= nil then
        if GameLib.GetPlayerUnit():IsMounted() then
            -- Mounted
            if GameLib.GetPlayerUnit():GetTarget() == nil then
                -- Mounted and Targeted
                strEmote = self:DecodeString(emote.untargeted)
            else
                -- Mounted and Untargeted
                strEmote = self:DecodeString(emote.targeted)
            end
        else
            -- Not mounted, so pass the command on to the chat interpreter as a command. Let it handle any errors
            -- Note that this means that, for now, the emote key (the part you type) has to be exactly the same as the built-in emote
            strEmote = self.kCommands.Prefix..strEmote
        end
        ChatSystemLib.Command(strEmote)
    else
        -- Error. They passed some text we don't understand
        self:ErrorOut(self.kMessages.Name.." "..self.kMessages.UnknownEmote..": '"..strEmote.."'")
    end
end

-----------------------------------------------------------------------------------------------
-- Internal methods
-----------------------------------------------------------------------------------------------
-- Using Realm messages for general info to the user
function MountSafe:MessageOut(str)
    ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Realm, str)
end

-- Debug channel for errors
function MountSafe:ErrorOut(str)
    ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Debug, self.kMessages.Name..": "..str)
end

-- Replaces substitution tokens with appropriate ones based on character target and gender
-- Also prepends "/emote " to make it a legal command. This also makes the character name
-- come out in a different color, be context-clickable, etc.
function MountSafe:DecodeString(strSource)
    local strRet = strSource

    -- Replace $t with the target name (if any)
    local target = GameLib.GetPlayerUnit():GetTarget()
    if target ~= nil then
        strRet = string.gsub(strRet, kRep.tar, target:GetName())
    end


    -- Get the possessive pronoun.
    local gender = GameLib.GetPlayerUnit():GetGender()

    --[[ Disabled for now. The game treats all Chua as male for the purpose of emotes, so sticking with that until I think of something better :-(
    -- Handle Chua as a special case.
    local race   = GameLib.GetPlayerUnit():GetRaceId()
    if race == kRaceChua then
        gender = kGender.Chua
    end
    --]]

    -- Replace $p with the possessive pronoun
    -- Not all languages need a pronoun based on gender
    local strPersonalPossessive = self.kPronouns.Possessive[gender] or ""
    strRet = string.gsub(strRet, kRep.pos, strPersonalPossessive)

    -- Prepend the emote command so it comes out as as close to what we want as we know how to get
    return self.kCommands.Prefix..self.kCommands.Emote.." "..strRet
end

-- Whitespace trimming function copied from Programming in Lua. It's a touch overkill since we "know" that left whitespace has been trimmed
-- by the command interpreter and it's a little odd because we aren't worrying about spaces within the parameters ("/ms wave at joe"), but it's passable.
function MountSafe:Trim(s)
  return (string.gsub(s, "^%s*(.-)%s*$", "%1"))
end

function MountSafe:ShowHelp()
    local strEmotes = table.concat(self.tEmotes, ", ")
    
    self:MessageOut(self.kMessages.Name..": ".. self.kMessages.Tagline.." "..self.kMessages.Version..kRep.nl..
                    self.kMessages.Usageline..kRep.nl..
                    self.kMessages.Supported..": "..strEmotes)
end

-----------------------------------------------------------------------------------------------
-- MountSafe Instance
-----------------------------------------------------------------------------------------------
local MountSafeInst = MountSafe:new()
MountSafeInst:Init()
